# Inlining variables

Author: Óscar Martín
Last modified: May 2021



Inlining is the process of transforming the use of variables, so that all variables that are declared and then used are *inlined* and the declarations removed. That is,
```maude
    var X : S .
    eq f(X) = X + 1 .
```
is transformed into
```maude
    eq f(X:S) = X:S + 1 .
```
The declaration is absent in the result.

In a philosophical sense, whether inlining is convenient or not has to do with whether variables are part of a module's signature, or are rather considered a necessary addition to a module, but not part of the signature. We use inlining when flattening a module into another. That is, transforming
```maude
mod B is
    inc A .
    <rest of B>
endm
```
into
```maude
mod B is
    <A's body>
    <rest of B>
endm
```
When an importation happens, Maude assumes that no clash of names occurs, that is, if function `f` is declared with that name in `A`, it is not defined in `B`. Well, it *can* be defined in both if the arities are different, but you get the point. So the question is, should we assume the same for variables, so as to avoid duplicate variable declarations? I think the answer is no. And, if I remember right, Maude thinks the same. Inlining variables before flattening is the solution. That is, instead of the above, the result of the flattening is
```maude
mod B is
    <A's body with variables inlined>
    <rest of B>
endm
```
