# Instantiation

Author: Óscar Martín
Last modified: May 2021



What I call *instantiation* is the creation of non-parameterized versions of parameterized elements: modules, instructions, or parts of them. We only care to instantiate `emod`s here: other types of modules we pass to Maude untouched.

The situation here is that an `emod` imports another `emod` with parameters:
```maude
emod ONE{X :: T1, Y :: T2} is
    sync X || Y
        on ...
endem

emod TWO is
    inc ONE{A, B} .
endem
```
(Assumed theories `T1` and `T2`, views `A` and `B`, and the modules involved are also present.) Our goal is to translate the `sync on` instruction into standard Maude, but we cannot do that unless first the formal arguments `X` and `Y` are fully instantiated using the views `A` and `B`. Indeed, all the contents of module `ONE` need to be instantiated. The instantiation consists, basically, in:
- replacing `X` and `Y` with the names of the modules specified in the views `A` and `B`;
- replacing the properties used in the syncing criteria with the ones mapped in the views;
- instantiating other instructions in `ONE`, that is, equations, memberships, and property inheritance instructions, other types of instructions are returned untouched (that is, a copy is returned).

The instantiation produces a new module called `<ONE|A|B>` and replaces the module `TWO` by
```maude
emod TWO is
    inc <ONE|A|B> .
endem
```

All `__instantiate` methods return new objects, that is, they do not modify their parameters. Even the public method `params_to_instances`, that is applied to a Maude top (that is, the syntax tree containing all modules and other Maude elements), returns a brand new top, with all its elements also brand new.

The only public method in `Instantiator` is `params_to_instances`. It looks inside `emod`s which import parameterized `emod`s (like `TWO` import `ONE` with parameters). For each such parameterized import (`inc ONE{A, B} .`), it produces the corresponding instantiated module (with name `<ONE|A|B>` in this case). Then it replaces the import by the one for the instantiation (`inc <ONE|A|B> .`). The module `ONE` as such is not included in the result. The module `TWO` is, with the modified importation, and still with the name `TWO`.

To instantiate a module, access is needed to theories, views, and modules, so the whole `MaudeTop` object is passed as parameter, and the whole new `MaudeTop` is returned.

The instantiation does nothing with membership assertions, just returns a copy of them untouched. At the moment, memberships are not accepted in `emod`s, so that's not a problem (we only instantiate `emod`s), but maybe we should accept them, and instantiate them when needed.

The instantiation procedure omits in the result any `emod` with parameters. The reason is that in such modules the arguments are always used in the `sync on` instruction. So no other use of parameters in `emod`s is supported.
