## On operator declarations

In Maude, the keywords `var` and `vars` are perfect synonyms. So are `sort` and `sorts`. However, `op` and `ops` are not. The following is a exploration of why.

The thing is that an operator identifier can be a complex entity, composed of several pieces. Thus, while the declaration `ops f g_ h : S -> S .` (note `ops`) declares three independent operators, the declaration `op f g_ h : S -> S .` (note `op` now) declares a single operator with a complex name. This operator is used, for example, as `f g X h` (for some variable `X`). For more down to Earth identifiers, `op` and `ops` can be used with the same effect: `op f : ...` and `ops f : ...` are both valid and equivalent. In particular, `ops` can be used to declare a unique operator. Also, the declaration `ops (f g_ h) : S -> S .` (note the parentheses) declares a unique operator and is exactly equivalent to `op f g_ h : S -> S .`

When breaking characters are involved in the names of operators, things become more complicated. For example, `op []_ : ...` declares a single operator composed of two parts (plus the argument), but `op ``[``]_ : ...` (double backquotes are a requirement of markdown here: they must be single ones in Maude code) declares an operator with a one-piece name. In both cases, however, the operators can be used indistinctly as `[]X`, or `[ ] X`, etc. and Maude understands. More tricky is using `ops` in these examples: `ops []_ : ...` declares three operators (`[`, `]`, and `_`), while `ops ``[``]_ : ...` declares only one. Then, `[]a` would be invalid in the first case (unless some empty-syntax juxtaposition operator is present), but is valid in the second case. Oddly enough, the second declaration allows the operator to be used with spaces: `[ ] a`. But backquotes are allowed (or are necessary) in the declaration, but not in the use.

Another detail to remember is that a backquote not followed by a breaking character represents a whitespace. Usually, it is needed in `ops` declarations, but not in `op` ones, neither outside declarations.

The precise way all this works, and when backquotes are needed, turns out to be difficult or odd in some extreme cases. Anyway, we only need to accept identifiers properly and let Maude process it.

This is something our implementation does not do properly. It considers `op` and `ops` as synonyms, and they expect a sequence of identifiers separated by whitespace, each one for an operator. This will change in the future.

---

2021-04
