# Pending tasks

This document contains a list of pending tasks on the implementation of *Compositional Maude*. It includes both the necessary, the slightly convenient, and anything in between. Also, it contains both small details and big changes and anything in between. It is a rather unstructured list. It is of value to the developers, probably not at all to users.








- This thing in progress:
    1. Modify the generation of the object model from the Maude text:
       - use names for arguments;
       - use the `entity(class)` thing;
       - *strip* all strings I receive from the parser, once and for all, so that I can remove all calls to `strip()` from `objmodel.py`;
       - etc.
    2. Generate the classes now in `objmodel.py` on the fly from `cmaude.lark` and `xlark.lark`. Adapt the thing with `entity(class)` so that my code understands it. Change them `cmaude.lark` to stop using `class___entity`.
    3. Write about all of this

- Tell git not to care about files named `kk*`.

- At some point, add `__repr__` methods to all classes in the object model.

- Maybe my *abstract* classes should have an `__init__` method that throws a `NotImplementedError` to prevent the creation of objects of those classes. The same for `__str__` and `__repr__`.

- I should be printing a `MaudeTop` instead of a collection of modules. Once I do it this way, I would be able to remove `\n\n` from the function `print_with_imports`, and also from `CondProcessor`.

- Maybe I should allow more that one new-line characters in formats. For instance, to separate modules.

- I should be able to add the module name in the messages that I write to the log file on the needed commands for verification.

- I should be able to remove the need for `//::-` in my grammars. I use it when there is only an element in the rhs of a rule, because then separators have no meaning. But I should be able to make the grammar in `xlark.lark` not require it. It is a question of designing that grammar, but I was not able when I tried.

- It would be better if, when I make explicit the name of the class corresponding to a BNF entity, I do it with the final capitalization right. At present I can't (or not easily), but I should be able in the future.

- I need to fix the thing with the optionals. Here is the problem: For `param_list`, I need it to be non-optional, because it is in the middle of a rule. But for `attr_list`, I need it to be optional, because otherwise Lark prefers `[...]` to be a term. (This seems to be the way Lark works.) At present, I have needed to include both possibilities in `xlark.lark`. When I modify the way I create the object model and work all the time in Python with arguments by name, I may unify both approaches, leaving `param_list` as optional when it appears. The solution would be to allow optional entities in sequential rules and create trivial values by default for them.

- Prepare two Lark grammars, one for cMaude, the other for normalized cMaude, and code a translator from the former to the latter. Modify (simplify) the current code so that it only works with the result of the translation. Then, prepare a third grammar, for standard normalized Maude, and refactor the current code so that it translates from grammars. Different object models would be needed for each grammar. Generate the object models based on the grammars. Write the result to file or generate classes for on-the-fly use as appropriate in each case. Study Python metaclasses ([here](https://realpython.com/python-metaclasses/), for example). See if I can produce all the object model on the fly, that is, including the definition of classes, based on the BNF input. Split `objmodel.py` in two, one with the automatically generated object model, the other with whatever additional methods are needed. In `ModBody`’s `__init__`, I am storing declarations one at a time. For example, `sorts S T .` is stored as two objects of the class `SortDecl`, one for the `S`, one for the `T`. However, the class `SortDecl` has as its attribute a *list* of sort identifiers. This may be needed in case I want to parse a sort declaration by itself (not as part of parsing a module). I’m not sure. I should analyze it. And maybe something similar is happening in other classes as well.

- Write down instructions on how the EBNF grammars must be in order for my program to behave correctly on them.

- Should I change `*args` to `List[args]` in all `objmodel.py`?

- Dice el libro de los profesores de TP1: "debe evitarse invocar métodos de un objeto miembro de otro objeto, que ha sido devuelto por una llamada a un método (por ejemplo un *getter*). Esto se puede resumir con el eslogan "usa un sólo punto", que se refiere al operador "." que se utiliza para invocar métodos: por ejemplo la sentencia `a.m1().m2()` rompe esta regla mientras que `a.m()` la respeta." Debería pensar en ello y aplicarlo a mi código.

- Change `AssignCondition` to `MatchCondition`, and other similar ones if there are any.

- Implement shortcuts or syntactic sugar as explained in the thesis text: accept general relations as syncing criteria; syntactic marks for in-place property definitions...

- Should I be able to remove all quotes from the types of arguments, after re-modulirising?

- See how to remove all methods from `Module`.

- The execution for the ABP example takes too long.

- Unit tests for class `Stdizer`. And for all others.

- The use of method `mod` in `CondProcessor` is ugly.

- The method `__inline_vars` for strings would be more reliable if coded using Maude's `metaParse` by means of Rubén's bindings.

- The instantiation procedure omits in the result any `emod` with parameters. The reason is that in such modules the arguments are always used in the `sync on` instruction. So no other use of parameters in `emod`s is supported.

- Method `instantiate` for `str` would be more reliable if coded using Maude's parser called through Rubén's tool, like `Qualifier` does.

- The instantiation is doing nothing with membership assertions, just returning a copy of them untouched. At the moment, memberships are not accepted in `emod`s, so that's not a problem (we only instantiate `emod`s), but maybe we should accept them, and instantiate them when needed.

- Change names of variables and functions to the ones with underscores, as Python advises. Ask the linter to tell me about this problem.

- Homogenize quotes: `"` or `'`.

- `inlineVarsInTerm` should be done by using RR's tool.

- Take care of op id appearing in `id:` attribute regarding qualification.

- Avoid using `_@_` literally in Python code, both in `Qualifier` and in other places.

- It seems the character `[` is not accepted in the id for an op, even if escaped. Anyway, I should make it possible to use any character in the declaration of an op, as Maude does (though Full Maude does not), and consider breaking chars there as if they were escaped.

- Allow `sync` without `on`, for cases when no criteria are needed.

- Improve everything having to do with declaration of operators. Especially, make `op` different to `ops`.

- Allow import instructions for several modules at once. That is, allow `pr M1 M2 .`

- Use this to allow longer lines in Lark rules:
```py
def joinlines(s):
    return re.sub(r'(?<=\S)\s*~\s*\r?\n\s*(?=\S)', r' ', s)
```

- In `Qualifier.__init__` I have `maude.load('model-checker.maude')`, because it is needed some times. But it is only needed in some cases, so it probably should not be always executed.

- Install newest Python version.

- See if I should detect ambiguities in parsing and inform the user when they happen, as Maude does.

- Use python's `import` in a finer way, so as to organize better all the code in `objmodel.py`. Read, for example, [Real Python on this](https://realpython.com/python-import).

- Learn about the `multiprocessing` Python module. Read [this](https://realpython.com/lessons/multiprocessing-module/), for instance.

- Some of my methods may not work well with operators declared using kinds instead of sorts.

- Avoid using `isinstance` so much.

- I’m including the syntax for formulas in the EBNF grammar, because I really need to be able to parse into formulas. But it is odd, because the syntax for formulas in not part of Maude syntax, but rather the signature declared in an imported module. They should be parsed in the second pass of a two-pass parse, if I had it. Doing it like I am doing it at present has the problem that formulas have a predefined syntax, and I cannot define my own formula operators, for instance, unless I add them to the EBNF definition. Also, this means that I cannot define auxiliary formulas, to be used in an `ag` statement, because their syntax is fixed in the grammar.

- I use sometimes `id` and sometimes `name` with the same purpose. Homogeneize.

- The definitions of `e_instruction` and `e_body` allow for more than one sync on instruction in a body. Avoid.

- The model checker and the tautology checker may be internally called by our tool, automating, in this way, all the process. (In this case, a `[proven]` attribute for `ag` statements may be useful, to tell the tool that that particular formula can be taken as good.)

- Create exceptions of appropriate types, instead of using `Exception` all the time. Also, use predefined exceptions that are fitting, like `NotImplementedError`.

- It may be useful to replace the entity `body` by `s_body`, `f_body`, `e_body`, and `a_e_body`.

- Allow labels for standard rules.

- A `GralRule` class may be useful, of which the `Rule` and also `CondRule`, `EgRule` and `CondEgRule` would be subclasses. Maybe even classes `GralEgRule` and `GralNonEgRule` could be useful to encompass conditional and unconditional ones.

- Operators to the kind must be declared by using the `~>`. The notation `[Kind]` is not accepted. Fix.

- I’m accepting rewrite conditions in equations and memberships. Fix.

- Problems notwithstanding, I should try to handle and translate correctly any atomic formula, like `p+3<5`, not only the ones that are just a Boolean property, as I am doing now.

- Review how I deal with rewrite conditions in rules. I must do nothing with them, but I’m not sure the process of simplification of conditions (in class `CondProcessor`) would not be broken if some rewrite condition is present.

- I need to instantiate parametric `emod`s, so that I can then perform the split on the result of the instantiation. I do not instantiate other kinds of modules: I leave it to Maude. However, I don’t know how well I instantiate if parameters are used for purposes other than their use in `sync on` and `inh` instructions.

- We do not accept parameterized theories, nor parameterized views, not `from` and `to` modules in views with parameters or mappings. Fix.

- Properties can be declared parametric, that is `ppt P : SomeSort -> OtherSort .` But, apart from the declaration, the translation probably does not work with them. Check.

- When simplifying rule conditions (in `reduceConds`), I’m assuming that all conditions are equational, assignment or Boolean ones.

- Having different subclasses of `ModBody` may be useful.

- When I produce a composed rule, in `compRulesFromSyncOn`, I add conditions to the rule for the origin composed state, and they can be present in the output. It is alright that I check them to see if the rule can be dropped. But it is not OK that they show in the output. If these conditions are false, the rule has not an lhs that is reachable, so the rule will never be used. If the rule come to be used, however, the condition is sure to be true, and checking it is wasting time.

- Maybe I should mention the existence of the `p` command line option. And make sure it works.

- My grammar only has a `mod_body` entity, defined as a list of `instruction`s. Thus, rules are welcome in functional modules. Fix.

- Use the trick about *monitoring how properties change*.

- When matching conditions result with the shape `X := ...`, for `X` a variable, the rule can be improved by replacing `X` in the text of the rule by the rhs and removing the condition.

- See if properties can be removed from systems when no longer needed. See what I mention at the end of the section about the sender in the example about ABP, for example.

- At present, `ag` statements can appear in theories, but the implementation does not care about them. If a module that instantiates a theory needs to assert that it satisfies the `ag`, it has to include the statement again in the body of the module. Maybe it should not be that way. Maybe the fact that a module instantiates a theory has to be enough for it to *inherit* the `ag`.

- It is a little odd that `ag` statements appear in the body of a module. A module is something executable; an `ag` statement is not. It may be more suitable that the assumption and the guarantee are declared in the module, but the assertion that they are satisfied… probably not.

- I should think seriously about implementing it all with a plugin architecture. There is some information and some pointers in a section of [Real Python on this](https://realpython.com/python-import). Allow modularity in the definitions and use of syntax extensions.

- Should I change all `__name` methods to `_name`?

- It seems now Lark accepts more features in the `regex` package... in case I still need them.

